-- ************************************************************************** --
-- * TitanSpec.lua                                                         
-- *                                                                        
-- * Original Author: Rothpada   
-- * Maintainer: Nasapunk88                                                       
-- ************************************************************************** --

TITAN_SPEC_ID = "TitanSpec";
TITANSPEC_VERSION = GetAddOnMetadata("TitanSpec", "Version") or "Unknown Version"
L = LibStub("AceLocale-3.0"):GetLocale("TitanSpec", true)
TITANSPEC_BUTTON_ICON = "Interface\\AddOns\\TitanSpec\\TitanSpec";
AceTimer = LibStub("AceTimer-3.0"); 
IS_DEBUGGING = ( UnitName("player") == "Balther" and GetRealmName() == "Shadow Council" )
						or ( UnitName("player") == "Rothpada" and GetRealmName() == "Broxigar" )
						or ( UnitName("player") == "Furrocious" and GetRealmName() == "Shadowsong" );
						
--Key Bindings
BINDING_HEADER_TITAN_SPEC = L["TITAN_SPEC_MENU_TEXT"];
BINDING_NAME_TITAN_SPEC_SPEC_GEAR = L["TITAN_SPEC_BINDINGS_SPEC_GEAR"];
BINDING_NAME_TITAN_SPEC_SPEC = L["TITAN_SPEC_BINDINGS_SPEC"];
BINDING_NAME_TITAN_SPEC_GEAR = L["TITAN_SPEC_BINDINGS_GEAR"];


TitanSpec_Icon = "";
TitanSpec_Equipped = "";
TitanSpec_BuildName = {"","",};
TitanSpec_RotateSet = 1;
TitanSpec_SwitchGear = true;

-- ************************************************************************** --
-- ******************************** Spec Change Chat Filter ************************* --
-- ************************************************************************** --
local f = CreateFrame"Frame" 

local orgSetActiveSpecGroup = _G.SetActiveSpecGroup 

local spam1 = gsub(ERR_LEARN_ABILITY_S:gsub("%.", "%."), "%%s", "(.*)") 
local spam2 = gsub(ERR_LEARN_SPELL_S:gsub("%.", "%."), "%%s", "(.*)") 
local spam3 = gsub(ERR_SPELL_UNLEARNED_S:gsub("%.", "%."), "%%s", "(.*)") 
local spam4 = gsub(ERR_LEARN_PASSIVE_S:gsub("%.", "%."), "%%s", "(.*)") 

local function SpamFilter(self, event, msg) 
    if strfind(msg, spam1) or strfind(msg, spam2) or strfind(msg, spam3) or strfind(msg, spam4) then return true end 
end 

function SetActiveSpecGroup(...) 
    f:RegisterEvent"UNIT_SPELLCAST_SUCCEEDED" 
    f:RegisterEvent"UNIT_SPELLCAST_STOP" 
    ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", SpamFilter) 
    return orgSetActiveSpecGroup(...) 
end 

f:SetScript("OnEvent", function(self, event, unit) 
    if unit ~= "player" then return end 
    self:UnregisterEvent"UNIT_SPELLCAST_SUCCEEDED" 
    self:UnregisterEvent"UNIT_SPELLCAST_STOP" 
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", SpamFilter) 
end)  															
								
-- ************************************************************************** --
-- ******************************** Titan Functions ************************* --
-- ************************************************************************** --

function TitanPanelTitanSpecButton_OnLoad(self)
	self.registry = {
		id = TITAN_SPEC_ID,
		version = TITANSPEC_VERSION,
		menuText = L["TITAN_SPEC_MENU_TEXT"],
		buttonTextFunction = "TitanPanelTitanSpecButton_GetButtonText",
		tooltipTitle = L["TITAN_SPEC_TOOLTIP_TITLE"],
		tooltipTextFunction = "TitanPanelTitanSpecButton_GetTooltipText",
		controlVariables = {
			ShowIcon = true,
			ShowLabelText = true,
			--ShowRegularText = false,
			--ShowColoredText = true,
			DisplayOnRightSide = true
		},
		savedVariables = {
			DisplayOnRightSide = 0,
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowColours = 1,
			ShowGearSet = 1,
			ShowSpec = 1,
			ShowHint = 1,
			ShowTooltipIcon = 1,
		}	
	};
	self:RegisterEvent("ADDON_LOADED");
    --self:RegisterEvent("PLAYER_LOGOUT");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	self:RegisterEvent("EQUIPMENT_SETS_CHANGED");
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");
	
	--TitanSpec_output(TITAN_SPEC_ID.." by Rothpada (Shadow Council)");
	TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
	TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL});
	AceTimer.ScheduleTimer(TITAN_SPEC_ID, function() 
            TitanSpec_refresh();
            TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL});
        end, 6);
end

function TitanPanelTitanSpecButton_OnClick(self, button)
	if ( button == "LeftButton" ) then
		if ( GetNumSpecGroups() > 1 ) then
				if ( IsShiftKeyDown() ) then
						Toggle_Spec();
				elseif ( IsAltKeyDown() ) then
						Toggle_Gear();
				else
						Toggle_Spec_Gear();
				end
		end
	end
end

function TitanPanelTitanSpecButton_OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and ... == "TitanSpec" then
		TitanSpec_EquipmentSet = TitanSpec_EquipmentSet or {"","",};
		TitanSpec_CustomLabel = TitanSpec_CustomLabel or {"","",};
		self:UnregisterEvent("ADDON_LOADED");
    elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		if ( TitanSpec_SwitchGear) then
			if ( TitanSpec_EquipmentSet ~= nil and 
					TitanSpec_EquipmentSet[GetActiveSpecGroup()] ~= nil and 
					TitanSpec_EquipmentSet[GetActiveSpecGroup()] ~= "" ) then
							UseEquipmentSet(TitanSpec_EquipmentSet[GetActiveSpecGroup()]);
							TitanSpec_Equipped = TitanSpec_EquipmentSet[GetActiveSpecGroup()];
			end
		else
			TitanSpec_SwitchGear = true;
		end
	elseif event == "EQUIPMENT_SETS_CHANGED" then
		CheckGearSets();
	end
	TitanSpec_refresh();
end

function TitanSpec_refresh()	
	--TitanSpec_debug("Refreshing...");
	if (TitanSpec_BuildName ~=nil or TitanSpec_BuildName ~= "") then
		TitanSpec_BuildName[1] = TitanSpec_getBuildName(1);
		TitanSpec_BuildName[2] = TitanSpec_getBuildName(2);
	end
	TitanSpec_getBuildName(0);
	TitanSpec_Equipped = TitanSpec_getEquipped();
	TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
end

function TitanSpec_getBuildName(index) 		
	local g = "";
	if index==0 then
		g = GetSpecialization();
	else
		g = GetSpecialization(false,false,index);
	end
	if ( g ~= nil ) then
		local _,specname,_,icon,_,_ = GetSpecializationInfo(g);
		if (index == 0) then
			TitanSpec_Icon = icon;
		end
		return specname;
	else 
		return "";
	end
end

function TitanSpec_getEquipped() 
	local numSets = GetNumEquipmentSets();
	for i=1,numSets do
		local name, _, _, isEquipped, _, _, _, _, _ = GetEquipmentSetInfo(i);
		if (isEquipped) then
			return name;
		end
	end
	return "";	
end

function TitanPanelTitanSpecButton_GetButtonText(id)
	local specNum = GetActiveSpecGroup();
	local label = L["TITAN_SPEC_SPEC"].." "..specNum..": ";
	local buildText = "";
	if ( not TitanGetVar(TITAN_SPEC_ID, "ShowLabelText") and not TitanGetVar(TITAN_SPEC_ID, "ShowSpec") and not TitanGetVar(TITAN_SPEC_ID, "ShowIcon") and not TitanGetVar(TITAN_SPEC_ID, "ShowLabelText") and not TitanGetVar(TITAN_SPEC_ID, "ShowGearSet") ) then
			buildText = L["TITAN_SPEC_MENU_TEXT"];	
	end
	
	if ( TitanGetVar(TITAN_SPEC_ID, "ShowSpec") ) then
				-- Build the button text	
			local customLabel = TitanSpec_CustomLabel;
			-- Fix for ticket 7
			if ( customLabel[specNum] == nil or type(customLabel[specNum]) == "boolean" ) then
					customLabel[specNum] = "";
					TitanSpec_CustomLabel = customLabel;
			end

			if ( customLabel == nil or strtrim(customLabel[specNum]) == "" ) then
				buildText = TitanSpec_getBuildName(0);
			else
				buildText = customLabel[specNum];
			end
	end
	
	if ( TitanGetVar(TITAN_SPEC_ID, "ShowGearSet") ) then
			if ( TitanSpec_Equipped ~= "") then
				if ( TitanSpec_EquipmentSet[GetActiveSpecGroup()] ~= nil and
					TitanSpec_EquipmentSet[GetActiveSpecGroup()] ~= "" and 
					TitanSpec_EquipmentSet[GetActiveSpecGroup()] ~= TitanSpec_Equipped) then
					buildText = format("%s [".."|cffff0000".."%s".."|cffffffff".."]", buildText, TitanSpec_Equipped);
				else
					buildText = format("%s [%s]", buildText, TitanSpec_Equipped);
				end
			else
				if ( not TitanGetVar(TITAN_SPEC_ID, "ShowLabelText") and 
					not TitanGetVar(TITAN_SPEC_ID, "ShowSpec") and 
					not TitanGetVar(TITAN_SPEC_ID, "ShowIcon") and 
					not TitanGetVar(TITAN_SPEC_ID, "ShowLabelText") ) then
					buildText = L["TITAN_SPEC_MENU_TEXT"];	
				end
			end
	end
	if ( TitanGetVar(TITAN_SPEC_ID, "ShowIcon") ) then
			local icon = "|T"..TitanSpec_Icon..":20|t";
	        buildText = format("%s %s", icon, buildText);
	end
	return label, TitanSpec_getColouredText(buildText, "white");
end

function TitanPanelTitanSpecButton_GetTooltipText(self)
	local g = GetSpecialization(false,false,1);
	local icon ="";
	if ( g ~= nil ) then
		 icon = select(4,GetSpecializationInfo(g));
	end
	local t1 = "";
	if (TitanGetVar(TITAN_SPEC_ID, "ShowTooltipIcon")) then
		t1 = " |T"..icon..":20|t "..L["TITAN_SPEC_PRIMARY_SPEC"];	
	else
		t1 = L["TITAN_SPEC_PRIMARY_SPEC"];
	end

	g = GetSpecialization(false,false,2);
	icon ="";
	if ( g ~= nil ) then
		icon = select(4,GetSpecializationInfo(g));
	end
	local t2 = "";
	if (TitanGetVar(TITAN_SPEC_ID, "ShowTooltipIcon")) then
		t2 = "\r".." |T"..icon..":20|t "..L["TITAN_SPEC_SECONDARY_SPEC"];
	else
		t2 = "\r"..L["TITAN_SPEC_SECONDARY_SPEC"];
	end
	
	if ( TitanSpec_CustomLabel[1] ~= nil and strtrim(TitanSpec_CustomLabel[1]) ~= "" ) then
		t1 = t1..TitanSpec_CustomLabel[1].." - ";
        else -- Fix for Ticket 8
                TitanSpec_CustomLabel[1] = "";
                TitanSpec_CustomLabel = TitanSpec_CustomLabel;
	end
	if ( TitanSpec_BuildName[1] ~= nil and strtrim(TitanSpec_BuildName[1]) ~= "" ) then
		t1 = t1..TitanSpec_BuildName[1];
	else
		t1 = t1.."?";
	end
	local icon1 = select(1,GetEquipmentSetInfoByName(TitanSpec_EquipmentSet[1]));
	local icon2 = select(1,GetEquipmentSetInfoByName(TitanSpec_EquipmentSet[2]));
	if ( TitanSpec_EquipmentSet[1] ~= nil and strtrim(TitanSpec_EquipmentSet[1]) ~= "" ) then
			if (TitanGetVar(TITAN_SPEC_ID, "ShowTooltipIcon")) then
				t1 = t1.."\n        ".."|T".."Interface\\Icons\\"..icon1..":19|t  "..TitanSpec_getColouredText(L["TITAN_SPEC_GEAR_SET"]..": "..TitanSpec_EquipmentSet[1], "white");	
			else
				t1 = t1.."\n        "..TitanSpec_getColouredText(L["TITAN_SPEC_GEAR_SET"]..": "..TitanSpec_EquipmentSet[1], "white");	
			end   
	end

	if ( GetNumSpecGroups() == 2 ) then
                -- Fix for ticket 7
                if ( TitanSpec_CustomLabel[2] == nil or type(TitanSpec_CustomLabel[2]) == "boolean" ) then
                        TitanSpec_CustomLabel[2] = "";
                        TitanSpec_CustomLabel = TitanSpec_CustomLabel;
                end
                if ( TitanSpec_BuildName[2] == nil or type(TitanSpec_BuildName[2]) == "boolean" ) then
                        TitanSpec_BuildName[2] = "";
                        TitanSpec_BuildName = TitanSpec_BuildName;
                end
                
		if ( strtrim(TitanSpec_CustomLabel[2]) ~= "" ) then
			t2 = t2..TitanSpec_CustomLabel[2].." - ";
		end
		if ( strtrim(TitanSpec_BuildName[2]) ~= "" ) then
			t2 = t2..TitanSpec_BuildName[2];
		else
			t2 = t2.."?";
		end
		if ( strtrim(TitanSpec_EquipmentSet[2]) ~= "" ) then
			if (TitanGetVar(TITAN_SPEC_ID, "ShowTooltipIcon")) then
				t2 = t2.."\n        ".."|T".."Interface\\Icons\\"..icon2..":19|t  "..TitanSpec_getColouredText(L["TITAN_SPEC_GEAR_SET"]..": "..TitanSpec_EquipmentSet[2], "white");
			else
				t2 = t2.."\n        "..TitanSpec_getColouredText(L["TITAN_SPEC_GEAR_SET"]..": "..TitanSpec_EquipmentSet[2], "white");
			end   
		end
		
		if ( TitanGetVar(TITAN_SPEC_ID, "ShowColours") ) then
			if ( GetActiveSpecGroup() == 1 ) then
				t1 = TitanSpec_getColouredText(t1, "green");
				t2 = TitanSpec_getColouredText(t2, "yellow");
			else
				t1 = TitanSpec_getColouredText(t1, "yellow");
				t2 = TitanSpec_getColouredText(t2, "green");
			end
		end
	else
		t2 = t2..L["TITAN_SPEC_NONE"];
		if ( TitanGetVar(TITAN_SPEC_ID, "ShowColours") ) then
			t1 = TitanSpec_getColouredText(t1, "green");
			t2 = TitanSpec_getColouredText(t2, "red");
		end
	end
	if (TitanGetVar(TITAN_SPEC_ID, "ShowHint")) then
		return "\r"..t1.."\r"..t2.."\r".."\r"..L["TITAN_SPEC_HINT"];	
	else
		return "\r"..t1.."\r"..t2;
	end
end

function TitanPanelRightClickMenu_PrepareTitanSpecMenu()
	--Level 2 Menu
	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		if ( UIDROPDOWNMENU_MENU_VALUE == "TITAN_SPEC_ASSIGN_EQUIPMENT_SET" ) then
				--Title
				local info = {};
				info.text = L["TITAN_SPEC_SPEC"].." 1 - "..TitanSpec_BuildName[1];
				info.notClickable = 1;
				info.isTitle = 1;
				info.notCheckable = true;
				UIDropDownMenu_AddButton(info, 2);
				
				--None
				info = {};
				info.text = "<"..L["TITAN_SPEC_NONE"]..">";
				info.func = function()
					TitanSpec_EquipmentSet[1] = "";
					TitanSpec_refresh();
				end
				info.checked = (TitanSpec_EquipmentSet[1] == "");
				UIDropDownMenu_AddButton(info, 2);
				
		        local numSets = GetNumEquipmentSets();
		        for i=1,numSets do
		                local setName = select(1,GetEquipmentSetInfo(i));
		                info = {};
						info.text = setName;
						if ( TitanSpec_EquipmentSet[1] == setName ) then
								info.text = "|cff00ff00"..info.text..FONT_COLOR_CODE_CLOSE;
								info.checked = 1;
						end
		                info.func = function()
                                    TitanSpec_EquipmentSet[1] = setName;
									if (GetActiveSpecGroup() == 1) then
										UseEquipmentSet(setName);
										TitanSpec_Equipped = setName;
									end
		                end
		                UIDropDownMenu_AddButton(info, 2);
		        end
		end
		if ( UIDROPDOWNMENU_MENU_VALUE == "TITAN_SPEC_ASSIGN_EQUIPMENT_SET_2" ) then
					--Title
					local info = {};
					info.text = L["TITAN_SPEC_SPEC"].." 2 - "..TitanSpec_BuildName[2];
					info.notClickable = 1;
					info.isTitle = 1;
					info.notCheckable = true;
					UIDropDownMenu_AddButton(info, 2);
					--None
					info = {};
					info.text = "<"..L["TITAN_SPEC_NONE"]..">";
					info.func = function()
						TitanSpec_EquipmentSet[2] = "";
						TitanSpec_refresh();
					end
					info.checked = (TitanSpec_EquipmentSet[2] == "");
					UIDropDownMenu_AddButton(info, 2);
					
					local numSets = GetNumEquipmentSets();
					for i=1,numSets do
							local setName = select(1,GetEquipmentSetInfo(i));
							info = {};
							info.text = setName;
							if ( TitanSpec_EquipmentSet[2] == setName ) then
									info.text = "|cff00ff00"..info.text..FONT_COLOR_CODE_CLOSE;
									info.checked = 1;
							end
							info.func = function()
										TitanSpec_EquipmentSet[2] = setName;
										if (GetActiveSpecGroup() == 2) then
											UseEquipmentSet(setName);
											TitanSpec_Equipped = setName;
										end
							end
							UIDropDownMenu_AddButton(info, 2);
					end
		end
        if ( UIDROPDOWNMENU_MENU_VALUE == "TITAN_SPEC_EQUIP_EQUIPMENT_SET" ) then
					local info = {};
					info.text = L["TITAN_SPEC_EQUIP_EQUIPMENT_SET"];
					info.notClickable = 1;
					info.isTitle = 1;
					info.notCheckable = true;
					UIDropDownMenu_AddButton(info, 2);
					info = {};
					local numSets = GetNumEquipmentSets();
					for i=1,numSets do
							local setName = select(1,GetEquipmentSetInfo(i));
							info = {};
							info.text = setName;
							info.notCheckable = true;
							info.func = function()
									UseEquipmentSet(setName);
									TitanSpec_Equipped = setName;
							end
							UIDropDownMenu_AddButton(info, 2);
					end
		end
		if (UIDROPDOWNMENU_MENU_VALUE == "TITAN_SPEC_BUTTON_TEXT") then
				local info = {};
				info.text = L["TITAN_SPEC_BUTTON_TEXT"];
				info.notClickable = 1;
				info.isTitle = 1;
					info.notCheckable = true;
				UIDropDownMenu_AddButton(info, 2);
				
				--Number
				local info = {};
				info.text = L["TITAN_SPEC_SHOW_NUMBER"];
				info.func = function()
					local label = not TitanGetVar(TITAN_SPEC_ID,"ShowLabelText");
					TitanSetVar(TITAN_SPEC_ID, "ShowLabelText", label);
					TitanSpec_refresh();
				end
				info.checked = TitanGetVar(TITAN_SPEC_ID,"ShowLabelText");
				info.keepShownOnClick = 1;
				UIDropDownMenu_AddButton(info, 2);

				--Icon
				local info = {};
				info.text = L["TITAN_SPEC_SHOW_ICON"]
				info.func = function()
					local icon = not TitanGetVar(TITAN_SPEC_ID,"ShowIcon");
					TitanSetVar(TITAN_SPEC_ID, "ShowIcon", icon);
					TitanSpec_refresh();
				end
				info.checked = TitanGetVar(TITAN_SPEC_ID,"ShowIcon");
				info.keepShownOnClick = 1;
				UIDropDownMenu_AddButton(info, 2);
				
				--Spec
				local info = {};
				info.text = L["TITAN_SPEC_SHOW_SPEC"]
				info.func = function()
					local spec = not TitanGetVar(TITAN_SPEC_ID,"ShowSpec");
					TitanSetVar(TITAN_SPEC_ID, "ShowSpec", spec);
					TitanSpec_refresh();
				end
				info.checked = TitanGetVar(TITAN_SPEC_ID,"ShowSpec");
				info.keepShownOnClick = 1;
				UIDropDownMenu_AddButton(info, 2);
				
				--GearSet
				local info = {};
				info.text = L["TITAN_SPEC_SHOW_GEAR_SET"]
				info.func = function()
					local gear = not TitanGetVar(TITAN_SPEC_ID,"ShowGearSet");
					TitanSetVar(TITAN_SPEC_ID, "ShowGearSet", gear);
					TitanSpec_refresh();
				end
				info.checked = TitanGetVar(TITAN_SPEC_ID,"ShowGearSet");
				info.keepShownOnClick = 1;
				UIDropDownMenu_AddButton(info, 2);
        end
		if (UIDROPDOWNMENU_MENU_VALUE == "TITAN_SPEC_TOOLTIP_TEXT") then
				--Title
				local info = {};
				info.text = L["TITAN_SPEC_TOOLTIP_TEXT"];
				info.notClickable = 1;
				info.isTitle = 1;
					info.notCheckable = true;
				UIDropDownMenu_AddButton(info, 2);
				
				--Color
				local info = {};
				info.text = L["TITAN_SPEC_SHOW_COLOURS"]
				info.func = function()
					local colors = not TitanGetVar(TITAN_SPEC_ID,"ShowColours");
					TitanSetVar(TITAN_SPEC_ID, "ShowColours", colors);
					TitanSpec_refresh();
				end
				info.checked = TitanGetVar(TITAN_SPEC_ID,"ShowColours");
				info.keepShownOnClick = 1;
				UIDropDownMenu_AddButton(info, 2);
				
				--Icons
				local info = {};
				info.text = L["TITAN_SPEC_SHOW_TOOLTIP_ICON"]
				info.func = function()
					local colors = not TitanGetVar(TITAN_SPEC_ID,"ShowTooltipIcon");
					TitanSetVar(TITAN_SPEC_ID, "ShowTooltipIcon", colors);
					TitanSpec_refresh();
				end
				info.checked = TitanGetVar(TITAN_SPEC_ID,"ShowTooltipIcon");
				info.keepShownOnClick = 1;
				UIDropDownMenu_AddButton(info, 2);

				--Hint
				local info = {};
				info.text = L["TITAN_SPEC_SHOW_HINT"]
				info.func = function()
					local hint = not TitanGetVar(TITAN_SPEC_ID,"ShowHint");
					TitanSetVar(TITAN_SPEC_ID, "ShowHint", hint);
					TitanSpec_refresh();
				end
				info.checked = TitanGetVar(TITAN_SPEC_ID,"ShowHint");
				info.keepShownOnClick = 1;
				UIDropDownMenu_AddButton(info, 2);
		end
		return;
	end
	-- Level 1 menu
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_SPEC_ID].menuText);
	TitanPanelRightClickMenu_AddSpacer();
	
	local info = {};
	info.text = L["TITAN_SPEC_RENAME"];
	info.notCheckable = true;
	info.func = function(self)
			StaticPopupDialogs["TITAN_SPEC_RENAME_DIALOG"] = {
			  text = L["TITAN_SPEC_RENAME_TEXT"],
			  button1 = L["TITAN_SPEC_ACCEPT"],
			  button2 = L["TITAN_SPEC_CANCEL"],
			  OnAccept = function (self, ...)
					local newName = self.editBox:GetText();
					local labels = TitanSpec_CustomLabel;
					labels[GetActiveSpecGroup()] = newName;
					TitanSpec_CustomLabel = labels;
					TitanSpec_refresh();
			  end,
			  timeout = 0,
			  whileDead = 1,
			  hideOnEscape = 1,
			  hasEditBox = 1,
			  enterClicksFirstButton = 1
			};
			StaticPopup_Show("TITAN_SPEC_RENAME_DIALOG");
	end
	UIDropDownMenu_AddButton(info);
	
	if ( GetNumEquipmentSets() > 0 ) then
		TitanPanelRightClickMenu_AddTitle(L["TITAN_SPEC_ASSIGN_EQUIPMENT_SET"], 2);
		info = {};
		info.text = L["TITAN_SPEC_ASSIGN_EQUIPMENT_SET"];
		info.value = "TITAN_SPEC_ASSIGN_EQUIPMENT_SET";
		info.notCheckable = true;
		info.hasArrow = 1;
		UIDropDownMenu_AddButton(info);
		
		TitanPanelRightClickMenu_AddTitle(L["TITAN_SPEC_ASSIGN_EQUIPMENT_SET_2"], 2);
		info = {};
		info.text = L["TITAN_SPEC_ASSIGN_EQUIPMENT_SET_2"];
		info.value = "TITAN_SPEC_ASSIGN_EQUIPMENT_SET_2";
		info.hasArrow = 1;
		info.notCheckable = true;
		UIDropDownMenu_AddButton(info);
		
		TitanPanelRightClickMenu_AddTitle(L["TITAN_SPEC_EQUIP_EQUIPMENT_SET"], 2);
		info = {};
		info.text = L["TITAN_SPEC_EQUIP_EQUIPMENT_SET"];
		info.value = "TITAN_SPEC_EQUIP_EQUIPMENT_SET";
		info.hasArrow = 1;
		info.notCheckable = true;
		UIDropDownMenu_AddButton(info);
		TitanPanelRightClickMenu_AddSpacer();
	end
	
	TitanPanelRightClickMenu_AddTitle(L["TITAN_SPEC_BUTTON_TEXT"], 2);
		info = {};
		info.text = L["TITAN_SPEC_BUTTON_TEXT"];
		info.value = "TITAN_SPEC_BUTTON_TEXT";
		info.notCheckable = true;
		info.hasArrow = 1;
		UIDropDownMenu_AddButton(info);
	
	TitanPanelRightClickMenu_AddTitle(L["TITAN_SPEC_TOOLTIP_TEXT"], 2);
		info = {};
		info.text = L["TITAN_SPEC_TOOLTIP_TEXT"];
		info.value = "TITAN_SPEC_TOOLTIP_TEXT";
		info.notCheckable = true;
		info.hasArrow = 1;
		UIDropDownMenu_AddButton(info);

	TitanPanelRightClickMenu_AddSpacer();
	info = {};
	info.text = "Close Menu";
	info.value = "Close Menu";
	info.hasArrow = false;
	info.notCheckable = true;
	UIDropDownMenu_AddButton(info, 1);
		
	TitanPanelRightClickMenu_AddSpacer();
	info = {};
	info.text = L["TITAN_SPEC_RESET"];	
	info.notCheckable = true;
	info.func = function()
			TitanSetVar(TITAN_SPEC_ID, "ShowLabelText", 1);
			TitanSetVar(TITAN_SPEC_ID, "ShowColours", 1);
		    TitanSetVar(TITAN_SPEC_ID, "ShowGearSet", 1);
			TitanSetVar(TITAN_SPEC_ID, "ShowIcon", 1);
			TitanSetVar(TITAN_SPEC_ID, "ShowSpec", 1);
			TitanSetVar(TITAN_SPEC_ID, "ShowHint", 1);
			TitanSetVar(TITAN_SPEC_ID, "ShowTooltipIcon", 1);
			TitanSpec_EquipmentSet = {"","",};
			TitanSpec_CustomLabel = {"","",};
			TitanSpec_refresh();
	end
	UIDropDownMenu_AddButton(info);
	TitanPanelRightClickMenu_AddCommand(L["TITAN_SPEC_HIDE"], TITAN_SPEC_ID, TITAN_PANEL_MENU_FUNC_HIDE);
end

function CheckGearSets()
	local numSets = GetNumEquipmentSets();
	if ( numSets < 1 ) then
		TitanSpec_EquipmentSet = {"","",};
	else 
		local has1 = false;
		local has2 = false;
		for i=1,numSets do
			local setName = select(1,GetEquipmentSetInfo(i));
			if ( TitanSpec_EquipmentSet[1] == setName) then
				has1 = true;
			end
			if ( TitanSpec_EquipmentSet[2] == setName) then
				has2 = true;
			end
		end
		if (not has1) then
			TitanSpec_EquipmentSet[1] = "";
		end
		if (not has2) then
			TitanSpec_EquipmentSet[2] = "";
		end
	end
end

--- Toggle functions for Mouse Clicks and Keybindings
function Toggle_Spec_Gear()
	local idx = mod(GetActiveSpecGroup(), GetNumSpecGroups()) + 1;
	SetActiveSpecGroup(idx);
end
function Toggle_Spec()
	local idx = mod(GetActiveSpecGroup(), GetNumSpecGroups()) + 1;
	TitanSpec_SwitchGear = false;
	SetActiveSpecGroup(idx);
end
function Toggle_Gear()
	if ( TitanSpec_EquipmentSet ~=nil and TitanSpec_EquipmentSet ~="" and
		TitanSpec_EquipmentSet[1] ~="" and TitanSpec_EquipmentSet[2] ~= "") then
		local set = nil;
		if ( TitanSpec_Equipped == TitanSpec_EquipmentSet[1]) then
			set = TitanSpec_EquipmentSet[2];
		elseif ( TitanSpec_Equipped == TitanSpec_EquipmentSet[2]) then
			set = TitanSpec_EquipmentSet[1];
		else
			set = TitanSpec_EquipmentSet[GetActiveSpecGroup()];
		end
		if ( set ~= nil and set ~= "" ) then
				UseEquipmentSet(set);
				TitanSpec_Equipped = set;
		end
	elseif (GetNumEquipmentSets() > 1) then 
		if (TitanSpec_RotateSet <= GetNumEquipmentSets()) then
			local name = select(1,GetEquipmentSetInfo(TitanSpec_RotateSet));
			if (name == TitanSpec_Equipped) then
				name = select(1,GetEquipmentSetInfo(TitanSpec_RotateSet+1));
			end
			if (name ~= nil and name ~= "") then
				UseEquipmentSet(name);
			end
			TitanSpec_Equipped = name;
			TitanSpec_RotateSet = TitanSpec_RotateSet+1;
		else
			TitanSpec_RotateSet = 1;
			local name = select(1,GetEquipmentSetInfo(TitanSpec_RotateSet));
			if (name == TitanSpec_Equipped) then
				name = select(1,GetEquipmentSetInfo(TitanSpec_RotateSet+1));
			end
			if (name ~= nil and name ~= "") then
				UseEquipmentSet(name);
			end
			TitanSpec_Equipped = name;
			TitanSpec_RotateSet = TitanSpec_RotateSet+1;
		end		
	elseif (GetNumEquipmentSets() == 1) then
		UseEquipmentSet(select(1,GetEquipmentSetInfo(1)));
	end
end
-- ************************************************************************** --
-- ******************************** User Functions ************************** --
-- ************************************************************************** --

function TitanSpec_debug(msg)
	if ( msg == nil ) then
		msg = "<nil>";
	end
	if IS_DEBUGGING then
		TitanDebug("TitanSpec> "..msg);
	end
end

function TitanSpec_output(msg)
    DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 1);
end

function TitanSpec_getColouredText(msg, colour)
	local colourCode = "";
	
	if ( colour == nil or colour == "default" ) then
		return msg;
	elseif ( colour == "red" ) then
		colourCode = "|cffff0000";
	elseif ( colour == "green" ) then
		colourCode = "|cff00ff00";
	elseif ( colour == "blue" ) then
		colourCode = "|cff0000ff";
	elseif ( colour == "yellow" ) then
		colourCode = "|cffffff00";
	elseif ( colour == "white" ) then
		colourCode = "|cffffffff";
	elseif ( colour == "black" ) then
		colourCode = "|cff000000";
	else
		colourCode = "|cff"..colour;
	end
        
	return colourCode..msg..FONT_COLOR_CODE_CLOSE;
end

